import { __awaiter, __generator } from "tslib";
import { MintLayout, TOKEN_PROGRAM_ID, Token, u64 } from "@solana/spl-token";
import { Keypair, PublicKey, SystemProgram, Transaction, } from "@solana/web3.js";
import bs58 from "bs58";
import { easyTokensRequest } from "../api/easyTokens";
import en from "../locales/en";
import vi from "../locales/vi";
import { checkIfTransactionSucceeded, createAssociatedTokenAccountInstruction, generateAssociatedTokenAccountAddress, } from "../utils/helpers";
/**
 * This hook returns:
 * - Initialize New Token Function
 * - Mint Token Function
 *
 * @param {WalletContextState | null} wallet: Use hook *useWallet* from '@solana/wallet-adapter-react' to retrieve
 *
 * @param {Connection} connection: Use hook *useConnection* from '@solana/wallet-adapter-react' to retrieve
 *
 * @param {string} locale: Can be "en" | "vi". Default is "en"
 *
 * @example
 *  const wallet = useWallet();
    const { connection } = useConnection();
    const { initializerToken, mintToken } = useEasyToken(
      wallet,
      connection,
      "en",
    );
 */
export var useEasyToken = function (wallet, connection, locale) {
    if (locale === void 0) { locale = "en"; }
    var trans = locale === "en" ? en : vi;
    var initializerToken = function (_a) {
        var data = _a.data, isMainnet = _a.isMainnet;
        return __awaiter(void 0, void 0, void 0, function () {
            var timestampMsg, signResult, mintAccount, payer, mintAuthority, decimalsChain, addressReceiveToken, lamports, userProgramAddressSync, userReceiveProgramAddressSync, transaction, totalSupply, _b, blockhash, lastValidBlockHeight, startInitTime, signature, txInitSucceeded, newTokenResult, error_1;
            var _c, _d;
            return __generator(this, function (_e) {
                switch (_e.label) {
                    case 0:
                        if (!wallet ||
                            !wallet.wallet ||
                            !wallet.publicKey ||
                            !wallet.signTransaction) {
                            return [2 /*return*/, {
                                    txSucceeded: false,
                                    error: trans.need_connect,
                                }];
                        }
                        _e.label = 1;
                    case 1:
                        _e.trys.push([1, 10, , 11]);
                        timestampMsg = new Date().toUTCString();
                        return [4 /*yield*/, signMessage(timestampMsg)];
                    case 2:
                        signResult = _e.sent();
                        if (!signResult.txSucceeded || !signResult.signedMessageBase58) {
                            return [2 /*return*/, { txSucceeded: false, error: signResult.error }];
                        }
                        mintAccount = Keypair.generate();
                        payer = wallet.publicKey;
                        mintAuthority = wallet.publicKey;
                        decimalsChain = data.decimals >= 0 && data.decimals <= 9 ? data.decimals : 9;
                        addressReceiveToken = new PublicKey(data.destinationAddress);
                        return [4 /*yield*/, Token.getMinBalanceRentForExemptMint(connection)];
                    case 3:
                        lamports = _e.sent();
                        return [4 /*yield*/, generateAssociatedTokenAccountAddress(mintAccount.publicKey, wallet.publicKey)];
                    case 4:
                        userProgramAddressSync = _e.sent();
                        return [4 /*yield*/, generateAssociatedTokenAccountAddress(mintAccount.publicKey, addressReceiveToken ? addressReceiveToken : wallet.publicKey)];
                    case 5:
                        userReceiveProgramAddressSync = _e.sent();
                        transaction = new Transaction();
                        transaction.add(SystemProgram.createAccount({
                            fromPubkey: payer,
                            newAccountPubkey: mintAccount.publicKey,
                            space: MintLayout.span,
                            lamports: lamports,
                            programId: TOKEN_PROGRAM_ID,
                        }));
                        transaction.add(Token.createInitMintInstruction(TOKEN_PROGRAM_ID, mintAccount.publicKey, data.decimals, mintAuthority, null));
                        transaction.add(createAssociatedTokenAccountInstruction(userProgramAddressSync, wallet.publicKey, wallet.publicKey, mintAccount.publicKey));
                        if (addressReceiveToken &&
                            (addressReceiveToken === null || addressReceiveToken === void 0 ? void 0 : addressReceiveToken.toBase58()) !== ((_c = wallet.publicKey) === null || _c === void 0 ? void 0 : _c.toBase58())) {
                            transaction.add(createAssociatedTokenAccountInstruction(userReceiveProgramAddressSync, wallet.publicKey, addressReceiveToken, mintAccount.publicKey));
                        }
                        totalSupply = new u64("".concat(data.supply * Math.pow(10, decimalsChain)));
                        transaction.add(Token.createMintToInstruction(TOKEN_PROGRAM_ID, mintAccount.publicKey, userReceiveProgramAddressSync, wallet.publicKey, [], totalSupply));
                        if (!data.canMint) {
                            transaction.add(Token.createSetAuthorityInstruction(TOKEN_PROGRAM_ID, mintAccount.publicKey, null, "MintTokens", mintAuthority, []));
                        }
                        return [4 /*yield*/, connection.getLatestBlockhash()];
                    case 6:
                        _b = _e.sent(), blockhash = _b.blockhash, lastValidBlockHeight = _b.lastValidBlockHeight;
                        transaction.recentBlockhash = blockhash;
                        transaction.feePayer = wallet.publicKey;
                        transaction.partialSign(mintAccount);
                        startInitTime = new Date();
                        return [4 /*yield*/, ((_d = wallet.wallet) === null || _d === void 0 ? void 0 : _d.adapter).sendTransactionWithSigners(transaction, connection, [mintAccount])];
                    case 7:
                        signature = _e.sent();
                        return [4 /*yield*/, checkIfTransactionSucceeded(connection, signature, lastValidBlockHeight, startInitTime)];
                    case 8:
                        txInitSucceeded = _e.sent();
                        if (!txInitSucceeded) {
                            return [2 /*return*/, {
                                    txSucceeded: false,
                                    error: trans.error.init_token,
                                }];
                        }
                        return [4 /*yield*/, easyTokensRequest({
                                signature: signResult.signedMessageBase58,
                                message: timestampMsg,
                                chain_name: data.chainName,
                                owner_address: wallet.publicKey.toBase58(),
                                coin_address: mintAccount.publicKey.toBase58(),
                                token_name: data.tokenName,
                                token_symbol: data.tokenSymbol,
                                decimals: data.decimals.toString(),
                                logo: data.logo,
                                description: data.description,
                                website: data.website,
                                gasless: signature ? true : false,
                                can_mint: data.canMint,
                                can_burn: true,
                                is_mainnet: isMainnet,
                            })];
                    case 9:
                        newTokenResult = _e.sent();
                        if (newTokenResult.error) {
                            return [2 /*return*/, {
                                    txSucceeded: false,
                                    error: newTokenResult.error,
                                }];
                        }
                        return [2 /*return*/, {
                                txSucceeded: true,
                                tx: signature,
                                mintAccount: mintAccount.publicKey.toBase58(),
                                mintBlock: lamports,
                                lastValidBlockHeight: lastValidBlockHeight,
                            }];
                    case 10:
                        error_1 = _e.sent();
                        console.error(error_1);
                        return [2 /*return*/, { txSucceeded: false, error: error_1 }];
                    case 11: return [2 /*return*/];
                }
            });
        });
    };
    var mintToken = function (_a) {
        var mintAccountAddress = _a.mintAccountAddress, amount = _a.amount, decimals = _a.decimals;
        return __awaiter(void 0, void 0, void 0, function () {
            var mintAddress, userProgramAddressSync, transaction, amountSupply, _b, blockhash, lastValidBlockHeight, startInitTime, signature, txMintSucceeded, error_2;
            var _c;
            return __generator(this, function (_d) {
                switch (_d.label) {
                    case 0:
                        if (!wallet || !wallet.publicKey || !wallet.signTransaction) {
                            return [2 /*return*/, {
                                    txSucceeded: false,
                                    error: trans.need_connect,
                                }];
                        }
                        _d.label = 1;
                    case 1:
                        _d.trys.push([1, 6, , 7]);
                        mintAddress = new PublicKey(mintAccountAddress);
                        return [4 /*yield*/, generateAssociatedTokenAccountAddress(mintAddress, wallet.publicKey)];
                    case 2:
                        userProgramAddressSync = _d.sent();
                        transaction = new Transaction();
                        amountSupply = new u64(amount * Math.pow(10, decimals));
                        transaction.add(Token.createMintToInstruction(TOKEN_PROGRAM_ID, mintAddress, userProgramAddressSync, wallet.publicKey, [], amountSupply));
                        return [4 /*yield*/, connection.getLatestBlockhash()];
                    case 3:
                        _b = _d.sent(), blockhash = _b.blockhash, lastValidBlockHeight = _b.lastValidBlockHeight;
                        transaction.recentBlockhash = blockhash;
                        transaction.feePayer = wallet.publicKey;
                        startInitTime = new Date();
                        return [4 /*yield*/, ((_c = wallet.wallet) === null || _c === void 0 ? void 0 : _c.adapter).sendTransactionWithSigners(transaction, connection, [])];
                    case 4:
                        signature = _d.sent();
                        return [4 /*yield*/, checkIfTransactionSucceeded(connection, signature, lastValidBlockHeight, startInitTime)];
                    case 5:
                        txMintSucceeded = _d.sent();
                        if (!txMintSucceeded) {
                            return [2 /*return*/, {
                                    txSucceeded: false,
                                    error: trans.error.mint_token,
                                }];
                        }
                        return [2 /*return*/, {
                                txSucceeded: true,
                                tx: signature,
                            }];
                    case 6:
                        error_2 = _d.sent();
                        console.error(error_2);
                        return [2 /*return*/, { txSucceeded: false, error: error_2 }];
                    case 7: return [2 /*return*/];
                }
            });
        });
    };
    var signMessage = function (message) { return __awaiter(void 0, void 0, void 0, function () {
        var encodedMsg, signedMessage, error_3;
        var _a;
        return __generator(this, function (_b) {
            switch (_b.label) {
                case 0:
                    if (!wallet || !wallet.publicKey || !wallet.signTransaction) {
                        return [2 /*return*/, {
                                txSucceeded: false,
                                error: trans.connect_demon_button.need_connect,
                            }];
                    }
                    _b.label = 1;
                case 1:
                    _b.trys.push([1, 3, , 4]);
                    encodedMsg = new TextEncoder().encode(message);
                    return [4 /*yield*/, ((_a = wallet.wallet) === null || _a === void 0 ? void 0 : _a.adapter).signMessage(encodedMsg)];
                case 2:
                    signedMessage = _b.sent();
                    return [2 /*return*/, {
                            txSucceeded: true,
                            signedMessageBase58: bs58.encode(Buffer.from(signedMessage)),
                        }];
                case 3:
                    error_3 = _b.sent();
                    console.error(error_3);
                    return [2 /*return*/, { txSucceeded: false, error: error_3 }];
                case 4: return [2 /*return*/];
            }
        });
    }); };
    return {
        initializerToken: initializerToken,
        mintToken: mintToken,
        signMessage: signMessage,
    };
};
//# sourceMappingURL=useEasyToken.js.map